#!/bin/sh\
exec wish "$0" "$@"

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

# ================= Help ============================================================
#
# Предоставляет:
#
#     proc showHelp {ihelp}
#     proc changesInHelpWindow {ihelp}
#
# Использует:
#
#     rawShowURL
#     proc setTitleHW    -- как устанавливается заголовок и иконка у окна.
#     global help        -- константы с параметрами вида окна и с help-ами
#                             help(bgButWin)  help(bgButAct) help(bgTextWin) -- цвета 
#                             help(titleLong) help(titleShort) help(windWidth) -- заголовки и ширина
#                             help(URLs) -- уникальный номер (freeindex)  
#                             help(I) help($i) help(N) help(currentHelp) -- стек и текущий топик
#                             help($help_name) -- топики
#     global myImages    -- картинки (для кнопок, если они графические)

proc startHelpWindow { } {
   global help
   global myImages

   if { [winfo exists .wHelp] } {
      return [ ]
   }
   toplevel .wHelp -background $help(bgButWin) -relief sunken -bd 4 -width 310 -height 400
   setTitleHW $help(titleLong) $help(titleShort)
   frame .wHelp.top -background $help(bgButWin) -width $help(windWidth)
   pack  .wHelp.top -side top -anchor w

   if { $help(textBtns) } {
      button .wHelp.top.bclose -text "Закрыть" -command "destroy .wHelp" -width 10 -bd 4
      button .wHelp.top.bindex -text "Оглавление" -command "showHelp HelpIndex" -width 10 -bd 4
      button .wHelp.top.blh    -text "<<<" -command "prevHelp" -width 10 -bd 4
      button .wHelp.top.brh    -text ">>>" -command "nextHelp" -width 10 -bd 4
      pack .wHelp.top.bclose .wHelp.top.bindex .wHelp.top.blh .wHelp.top.brh -side left -pady 3 -padx 3 -fill x
   } else {
      setImgButtons .wHelp.top 2 2 left w [subst {
          close  $myImages(noNormal)    {destroy .wHelp}
          index  $myImages(helpNormal)  {showHelp HelpIndex}
          lh     $myImages(prevNormal)  prevHelp
          rh     $myImages(nextNormal)  nextHelp
      }]
   }

   frame .wHelp.delim -width $help(windWidth) -height 3 -borderwidth 1 -relief sunken -background $help(bgTextWin) 
   pack  .wHelp.delim -side top -pady 2 -padx 1
   frame .wHelp.cntnt -background $help(bgTextWin) 
   pack  .wHelp.cntnt -side top -fill both -expand yes
   text .wHelp.cntnt.text -relief sunken -bd 2  -background $help(bgTextWin) -font {Helvetica 10} \
        -yscrollcommand ".wHelp.cntnt.scroll set" -setgrid 1 -height 30 -width 20 -wrap word
   scrollbar .wHelp.cntnt.scroll -command ".wHelp.cntnt.text yview"
   pack .wHelp.cntnt.scroll -side right -fill y
   pack .wHelp.cntnt.text -expand yes -fill both
   foreach {name parms} \
      { H1 "-font {Helvetica 13 bold} -foreground blue4"
        N  "-font {Helvetica 10}"
        B  "-font {Helvetica 10 bold}"
        S  "-font {Helvetica 5}"
        I  "-font {Helvetica 10 italic}"
        EM "-font {Helvetica 10 bold italic}"
        H2 "-font {Helvetica 11 bold} -foreground blue4"
        IT "-font {Helvetica 10 bold} -foreground DeepSkyBlue4"
      } {
      eval [subst ".wHelp.cntnt.text tag configure $name $parms -lmargin1  5 -lmargin2  5 -rmargin 5"]
      eval [subst ".wHelp.cntnt.text tag configure L1$name $parms -lmargin1 15 -lmargin2 30 -rmargin 5"]
      eval [subst ".wHelp.cntnt.text tag configure L2$name $parms -lmargin1 30 -lmargin2 45 -rmargin 5"]
   }
}
#
proc helpOnTop { } {
   global help
   set i $help(I)

   if { $i > 0 } {
      catch {.wHelp.top.blh configure -state normal}
   } else {
      catch {.wHelp.top.blh configure -state disable}
   }
   if { ($i+1) < $help(N)} {
      catch {.wHelp.top.brh configure -state normal}
   } else {
      catch {.wHelp.top.brh configure -state disable}
   }
   catch {wm deiconify .wHelp}
   catch {focus -force .wHelp}
   update
}
#
proc changesInHelpWindow {ihelp} {
   global help
   if { [winfo exists .wHelp] } {
      showHelp $ihelp
   }
}
#
proc showHelp {ihelp} {
   global help

   if { [string first "http://" $ihelp] != -1 } {
      rawShowURL $ihelp
      return []
   }

   if { [winfo exists .wHelp] && ($ihelp eq $help(currentHelp)) } {
      helpOnTop
      return []
   }
   startHelpWindow
   set  i       $help(I)
   incr i
   set help($i) $ihelp
   set help(I)  $i
   set help(N)  [expr $i+1]
   refreshHelp  $ihelp
}
#
proc prevHelp { } {
   global help
   if { ![winfo exists .wHelp]} {
      return []
   }
   set i        [expr $help(I)-1]
   if { $i >= 0 } {
      set help(I)  $i
      refreshHelp  $help($i)
   }
}
#
proc nextHelp { } {
   global help
   if { ![winfo exists .wHelp]} {
      return []
   }
   set i        [expr $help(I)+1]
   if {$i<$help(N)} {
      set help(I)  $i
      refreshHelp  $help($i)
   }
}
#
proc refreshHelp {ihelp} {
   global help
   set tisc $help($ihelp)
   setTitleHW [lindex $tisc 0] [lindex $tisc 1]
   .wHelp.cntnt.text configure -state normal
   .wHelp.cntnt.text delete 0.0 end
   .wHelp.cntnt.text configure -state disabled -cursor arrow
   insertHelp [lindex $tisc 2]
   set help(currentHelp) $ihelp
   helpOnTop
   update
}
#
proc insertHelp { x } {
   global help
   global myImages
   set active_url "-underline on -foreground red2 -font {Helvetica 10 bold}  -lmargin1 5 -lmargin2 5 -rmargin 5"
   set normal_url "-underline on -foreground blue2 -font {Helvetica 10 bold}  -lmargin1 5 -lmargin2 5 -rmargin 5"
   set ts  [split $x "`"]
   set len [llength $ts] 
   for {set i 0} {$i<$len} {incr i} {
      set t [lindex $ts $i]
      set tag N
      incr i
      if {$i<$len} { 
         set tag [lindex $ts $i]
      }
      if { [string first "`$tag`"\
            "`IMGB`H1`B`N`I`EM`H2`IT`L1H1`L1B`L1N`L1I`L1EM`L1H2`L1IT`L2H1`L2B`L2N`L2I`L2EM`L2H2`L2IT`HREF`"] == -1 } {
         set tag N
      }

      if {$tag eq "IMGB"} {
         set  j $help(URLs)
         incr j
         set help(URLs) $j
         .wHelp.cntnt.text configure -state normal
         catch {
            button .wHelp.cntnt.text.b$j -image $myImages($t) -command { }  -cursor arrow \
                -background $help(bgButWin) -activebackground $help(bgButAct) -highlightbackground $help(bgButAct)
            .wHelp.cntnt.text window create end -window .wHelp.cntnt.text.b$j -padx 6 -pady 2 
         }
         .wHelp.cntnt.text configure -state disabled -cursor arrow
      } else {
         if {$tag eq "HREF"} {
            set url $help(currentHelp)
            incr i
            if {$i<$len} { 
               set url [lindex $ts $i]
            }
            set  j $help(URLs)
            incr j
            set help(URLs) $j
            set tag "$tag$j"
            catch [subst ".wHelp.cntnt.text tag configure $tag $normal_url"]
            .wHelp.cntnt.text tag bind $tag <Any-Enter> ".wHelp.cntnt.text tag configure $tag $active_url"         
            .wHelp.cntnt.text tag bind $tag <Any-Leave> ".wHelp.cntnt.text tag configure $tag $normal_url"
            .wHelp.cntnt.text tag bind $tag <1> "showHelp $url"         
         }
         .wHelp.cntnt.text configure -state normal
         .wHelp.cntnt.text insert end $t $tag
         .wHelp.cntnt.text configure -state disabled -cursor arrow
      }
   }
}
