#!/usr/bin/env wish

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

# ================= Общие базовые процедуры (форматеры и т.п.) ==================
#
# Tcl часть:
#
# proc rawPing {host {size 1450} {timeout 250} }
#                                      -- возвращает список
#                                         {rec (0:1) ms $msec lastPingCmd $pingcmd lastPingRes $pres}
#                                         Удобно пользовать так     array set xxx [rawPing $host $size $timeout]
#     proc pswdToKey { pswd }          -- перевод пароля в ключ
#     proc chkIP { ip }                -- строка есть нормальный IP-номер?
#     proc scolSplit { text {bls ""} } -- разбить $text на непустые токены по ":", 
#                                         заменить (regsub -all) $bls на пробелы
#     proc readTokensFromFile {file}   -- чтение файла, разбивка его по токенам, возвращает список токенов
#                                         Про ошибку: переменные bad_file_or_token (0 | 1) и bad_file_or_token_msg
#     proc checkToken { condition msg }-- Проверка условия на токен, запись (в bad_file_or_token и bad_file_or_token_msg) 
#                                         текста про ошибку, если токен не верный
#     proc leftDots  {len str}
#     proc rightDots {len str}
#     proc limits { x xmin xmax }
#     proc min { a b }
#     proc max { a b }
#     proc strTime { }
#     proc wrapLines { len s }         -- текст (без \n) --> в текст
#     proc splitWrapLines { len txt }  -- текст с \n и с " // " --> в список текстов
#     proc getWhoAmIInfo {}            -- данные сервиса WhoAmI
#
# Tk часть:
#
#     proc message { tag str }         -- $tag == error question warning info 
#     proc showMessage { tag txt }     -- длинные информационные сообщения
#     proc setImgButtons { w padx pady side bs }
#     proc rawShowURL { url }
#
# Процедуры асинхронного чтения по HTTP:
# 
#     proc readURL { url timeout }  
# 
#                  порядок использования: 
# 
#                  (1) В аппликации надо описать процедуру 
# 
#                         proc analyzeDataFromUrl { url data }
# 
#                  (2) Вызываем 
# 
#                         readURL url timeout
# 
#                  (3) Если (и когда) будут без ошибок считаны данные data, то
#                      будет вызвана процедура 
# 
#                         analyzeDataFromUrl url data
# 
#                      Не плохая идея в этой процедуре сделать switch по url.  
#                      И помнить про переадресацию -- прочитаться может совсем 
#                      не тот url, что мы просили! ...


# ================= Tcl часть: ==================================================
proc rawPing {host {size 1450} {timeout 250} } {
# возвращает список { rec (0:1) ms $msec lastPingCmd $pingcmd lastPingRes $pres}
#
# Удобно пользовать так        array set xxx [rawPing $host $size $timeout]
#
# реализация системозависимаясейчас поддержаны любые Windows, я надеюсь.  Идея и реализация Е.Куршева

   if {[windowsPlatform]} {
   set pingregexpr {TTL=.*Ping.*=\s*\d+.*=\s*(\d+).*=\s*\d+.*=\s*\d+.*=\s*\d+.*=\s*(\d+)}
#                               Send^^^    Rcv^^^   Lost^^^   Min^^^   Max^^^    Avg^^^
   } else {
       set pingregexpr {ms.*\d+.*,\s*(\d+).*,\s*\d+.*=\s*\d+\.\d+/(\d+)\.\d+/\d+.*}
#                      Send^^^    Rcv^^^    Lost^^^   Min^^^^^^^Avg^^^^^^^Max^^^    
   }

   set rec 0
   set ms  0
   set pingres 0
   if {[windowsPlatform]} {
	   if {[WinNT6]} {
    	   set pingcmd "ping -4 -n 1 -l $size -w $timeout $host"
	   } else {
	       set pingcmd "ping -n 1 -l $size -w $timeout $host"
	   }
   } else {
       set pingcmd "ping -c 1 -s $size -i [expr $timeout/1000.0] $host"
   }
   catch "exec $pingcmd" pingres

   regexp $pingregexpr $pingres skip rec ms
   set rec [limits $rec 0   1]
   set ms  [limits $ms  0 $ms]

   set pingres [encoding convertfrom cp866 [encoding convertto cp1251 $pingres]]
   regsub -all "\n\n" $pingres "\n" pres
   return "rec $rec ms $ms lastPingCmd {$pingcmd} lastPingRes {$pres}"
}
#                      -------- перевод пароля в ключ
proc pswdToKey { pswd } {
   return [string tolower [md5::md5 -hex -- [encoding convertto koi8-r $pswd]]]
}
#                      -------- проверка синтаксиса строки, как IP-номера
proc chkIP { ip } {
   set x {^([1-2][0-9][0-9]|[1-9][0-9]|[0-9]).([1-2][0-9][0-9]|[1-9][0-9]|[0-9]).([1-2][0-9][0-9]|[1-9][0-9]|[0-9]).([1-2][0-9][0-9]|[1-9][0-9]|[0-9])$}
   set a -1
   set b -1
   set c -1
   set d -1
   return [expr [regexp $x $ip skip a b c d]\
                &&  (-1 < $a) && ($a < 256) \
                &&  (-1 < $b) && ($b < 256) \
                &&  (-1 < $c) && ($c < 256) \
                &&  (-1 < $d) && ($d < 256) \
          ]
}
#                      -------- разбить $text на непустые токены по ":", 
#                               заменить (regsub -all) $bls на пробелы
proc scolSplit { text {bls ""} } {
   set t $text

# WhoAmI возвращает Time в виде "гггг/мм/дд чч:мм:сс", а клиент раньше думал,
# что в формате ггггммддччммсс. Поэтому в отчёте не было слов
# "Точное время (с сервера):... Локальное время (у Абонента)".
# Ошибка теперь исправлена, но пришлось изменить эту процедуру (scolSplit)
# до неузнаваемости.

    set tokens [split $t \n]
    set list   [list]
    # 1. Удаление пустых строк:
    foreach el $tokens {
        if {[llength $el] > 0} {
            lappend list $el
        }
    }
    set list2 [list]
    foreach el $list {
        set tmp $el
        # 2. Удаление ":" в начале строк:
        if {[string index $el 0] eq ":"} {
            set tmp [string range $tmp 1 end]
        }
        # 3. Удаление ":" в конце строк:
        if {[string index $tmp end] eq ":"} {
            set tmp [string range $tmp 0 end-1]
        }
        # 4. Если встретили параметр Time, то его значение
        # аккуратно оформляется в виде списка (в фигурные скобки):
        set spl [split $tmp :]
        if {[llength $spl] > 2} {
            set par [lindex $spl 0]
            set len [string length $par]
            if {$par eq "Time"} {
                set val [string range $tmp [expr $len+1] end]
                lappend list2 $par $val
            } else {
                foreach elem $spl {
                    lappend list2 $elem
                }
            }
        } else {
            lappend list2 [lindex $spl 0] [lindex $spl 1]
        }
    }
    return $list2    

   regsub -all " |\t|\n" $t {} x

   # заменим $bls на " ":
   if { [string length $bls] > 0 } {
      regsub -all $bls $x " " x
   }

   set tokens [split $x :]

   # Отбросим пустые токены везде:
   set i [lsearch -exact $tokens {}]
   while { ([llength $tokens] > 0) && ($i > -1) } {
      set tokens [lreplace $tokens $i $i]
      set i [lsearch -exact $tokens {}]
   }
   return $tokens
}
#                      -------- чтение файла, разбивка его по токенам, возвращает список токенов
#                               Про ошибку: переменные bad_file_or_token (0 | 1) и bad_file_or_token_msg
#
proc readTokensFromFile {file} {
   global bad_file_or_token
   global bad_file_or_token_msg

   set bad_file_or_token     0
   set bad_file_or_token_msg ""

   set ok 0
   catch {
      set f [open $file r]
      set ping(file) {}
      set t ""     
      while {![eof $f]} {
          set t "$t[read $f 10000]"
      }
      close $f
      set ok 1
   } rc
   if { ! $ok } {
      checkToken 0 "Error: [mkErrorCode $rc]"
      return {}
   }
   return [scolSplit $t]
}
#                      -------- Проверка условия на токен, запись (в bad_file_or_token и bad_file_or_token_msg) 
#                               текста про ошибку, если токен не верный
proc checkToken { condition msg } {
   global bad_file_or_token
   global bad_file_or_token_msg

   if { ($bad_file_or_token==1) || ($condition==1) } { 
      return [] 
   }
   set bad_file_or_token 1
   set bad_file_or_token_msg $msg
   return []
}
#                      -------- наложить на x ограничения xmin ... xmax
proc limits { x xmin xmax } {
   return [expr ( $x < $xmin )  ? $xmin : ( $x > $xmax ? $xmax : $x ) ]
}
#                      -------- min
proc min { a b } {
   return [expr ($a < $b)  ? $a  : $b ]
}
#                      -------- max
proc max { a b } {
   return [expr ($a > $b)  ? $a  : $b ]
}
#                      -------- информационные сообщения, $tag == error question warning info 
proc message { tag str } {
   set ttl(info)     "Сообщение"
   set ttl(warning)  "Предупреждение"
   set ttl(error)    "Ошибка"
   set ttl(question) "Вопрос"
   if {![lcontain {error info question warning} $tag]} {
      set title ""
   } else {
      set title $ttl($tag)
   }
   tk_messageBox -icon $tag -type ok -message $str -title $title
}
#                      -------- длинные информационные сообщения, $tag == error question warning info 
proc showMessage { tag txt } {
   message $tag [join [splitWrapLines 80 $txt] "\n"]
   return []
}
#                      -------- текст с текущими датой и временем
proc strTime { } {
   return [clock format [clock seconds] -format "%d/%m/%Y %H:%M:%S"]
}
proc strTime2 { } {
   return [clock format [clock seconds] -format %Y%m%d%H%M%S]
}
#                      -------- текст (без \n) --> в текст
proc wrapLines { len s } {
   set s [string trimright $s]
   set t ""
   set i1 [expr $len - 1]
   while { [string length $s] > $len } {
      set s1 [string range $s 0 $i1]
      set s3 [string range $s $len end]
      set j  [string last " " $s1]
      if { $j == -1 } {
         set t "$t$s1\n"
         set s $s3
      } else {
         set s11 [string range $s1 0 [expr $j - 1]]
         set s2  [string range $s1 [expr $j + 1] end]
         set t "$t$s11\n"
         set s "$s2$s3"
      }
   }
   set t "$t$s\n"
   return $t
}
#                      -------- текст с \n и с " // " --> в список текстов
proc splitWrapLines { len txt } {
   regsub -all " // " $txt  "\n" txt1
   regsub -all "\n\n" $txt1 "\n" text

   # Список абзацев:
   set pars [split $text "\n"]

   # Отбросим пустые абзацы везде:
   set i [lsearch -regexp $pars {^\s*$}]
   while { ([llength $pars] > 0) && ($i > -1) } {
      set pars [lreplace $pars $i $i]
      set i [lsearch -exact $pars {}]
   }

   # Форматируем все абзацы и вставляем пустые строки между ними.
   set f 1
   set lines ""
   foreach par $pars {
      set p_lines [wrapLines $len [string trimright $par ] ]
      if { $f } {
         set lines $p_lines
      } else {
         set lines "$lines\n$p_lines"
      }
      set f 0
   }
   set lines [split $lines "\n"]
   set n [llength $lines] 
   if { $n > 1 } {
      set lines [lrange $lines 0 [expr $n - 2]]
   }
   return $lines
}
#                      -------- сделать строку не длиннее len; обрезка и "..." слева
proc leftDots {len str} {
   set l [string length $str]
   if {$l <= $len} {
      return $str; 
   }
   return "...[string range $str [expr $l+3-$len] end]"
}
#                      -------- сделать строку не длиннее len; обрезка и "..." справа
proc rightDots {len str} {
   set l [string length $str]
   if {$l <= $len} {
      return $str;
   }
   return "[string range $str 0 [expr $len-4]]..."
}

# ================= Tk часть: ===================================================

proc setImgButtons { w padx pady side a bs } {
   foreach {n img cmd} $bs {
      button $w.b$n -image $img -command $cmd \
             -background gray84 -activebackground LightSteelBlue -highlightbackground LightSteelBlue
      pack $w.b$n -expand 1 -padx $padx -pady $pady -side $side -anchor $a
   }
   return [ ]
}

#                      -------- rawShowURL базовая операция, реализация системозависимая
#
# сейчас поддержаны любые Windows, я надеюсь.  Идея про $env(COMSPEC) --- Е.Куршева
# идея использования "tmp.url" (С.Абрамов) решила (я надеюсь) проблему со странной 
# паузой запуска браузера
# 
# запускает окно браузера на указанный URL.
#

proc rawShowURL { url } {
    # On Win95, even if there is a web browser installed, it cannot
    # open an internet address, only a local .html file.
    if {[lindex [array get ::tcl_platform] 1]=="4.0"} {
        message info "Ваша операционная система ($::tcl_platform(os)) не поддерживает такой переход по ссылке.\nАдрес ссылки: $url"
        return
    }
    # Win98 doesn't like "/":
    if {[lindex [array get ::tcl_platform] 1]=="4.10"} {
        regsub -all "/" $url "\\\\" url
    }
    # Substitute & with ^&.
    # (Example: http://www.ideogramic.com?a=1&b=1 => http://www.ideogramic.com?a=1^&b=1
    # Otherwise, it will only open http://www.ideogramic.com?a=1).
    regsub -all "&" $url "^&" url
    # Open URL
    if {[catch {
        eval exec [auto_execok start] [list $url] &
    } rc]} {
        message error "Не удалось отобразить URL\n$url\n\nError: [mkErrorCode $rc]"
        asksendmail "Не удалось отобразить URL\n$url\n\nError: [mkErrorCode $rc]"
    }

    return

    if {[catch {
        if {[string match *.htm $url]} {
            set url [regsub -nocase {\.htm} $url {\.ht%6D}]; # rundll32 не умеет работать с htm-файлами, поэтому нужно хитрить
        }
        exec rundll32 url.dll,FileProtocolHandler $url &
    } rc]} {
        message error "Не удалось отобразить URL\n$url\n\nError: [mkErrorCode $rc]"
        asksendmail "Не удалось отобразить URL\n$url\n\nError: [mkErrorCode $rc]"
    }
    
    return

   global env

   set x $env(COMSPEC)
   if {[string length $x] == 0} {
      message info "В Вашей операционной системе не задана\nпеременная окружения COMSPEC."
      return []
   }
   set ok 0
   catch {
      set tmpfile [tempFile tmp.url]
      set f [open $tmpfile w]
      puts $f "\[InternetShortcut\]\nURL=$url"
      set ok 1
   } rc
   if { $ok } { 
      catch { close $f }
      catch { exec $x /c start $tmpfile }
   } else {
      catch { close $f }
      set msg "Не удалось отобразить URL\n$url\n\nError: [mkErrorCode $rc]"
      message error $msg
      asksendmail $msg
   }
   return []
}

proc tempFile { fname } {
   global env
   set d {}
   catch { set d $env(TEMP) }
   if {[string length $d] == 0} { catch { set d $env(TMP) } }
   set tmpfile $fname
   if {[string length $d] != 0} {
      set tmpfile [file join $d $fname]
   }
   return $tmpfile
}

proc appFile {fname {curdir 0}} {
   if {[windowsPlatform]} {
	  if {$curdir} {return $fname}
      if {[info exists ::env(APPDATA)]} {
         set dir $::env(APPDATA)
      } else {set dir {}}
      set folder "BotikTools"
   } else {
	  set dir    $::env(HOME)
      set folder ".BotikTools"
	  if {$curdir} {return [file join $folder $fname]}
   }
   set appFile $fname
   if {![lempty $dir]} {
      catch {
         set appPath [file join $dir $folder]
         file mkdir $appPath
         set appFile [file join $appPath $fname]
		 file attributes $appPath -permissions 0700
      }
   }
   return $appFile
}

# proc rawShowURL { url } {
#    global env
#    set x $env(COMSPEC)
#    if {[string length $x] == 0} {
#       message warning "В Вашей операционной системе не задана\nпеременная окружения COMSPEC."
#       return []
#    }
#    message warning "Доступ к странице $url\nможет занять некоторое время.\n\nБудьте терпеливы."
#    after 100
#    catch {exec $x /c start $url} skip
#    return []
# }

# ================= Процедуры асинхронного чтения по HTTP: =========================

package require http 2.4 
package require tls  1.4 
http::register https 443 ::tls::socket

proc readURL {url timeout} {
   catch {
      ::http::geturl $url -command chkReadUrl -timeout $timeout
   }
   return
}

proc chkReadUrl {token} {
   upvar #0 $token state

   set err  ok
   set data ""
   set url  ""
   catch { 
      set err $state(error)
   }

   catch { 
      set rc "<$err><$state(status)><$state(http)>"
      if {$rc eq "<ok><ok><HTTP/1.1 200 OK>"} {
         set data $state(body)
         set url $state(url)
         analyzeDataFromUrl $url $data
      } elseif {$rc eq "<ok><ok><HTTP/1.1 302 Moved>"} {
         array set meta $state(meta)
         after 500 readURL $meta(location) 5000
      }
   }
   catch { 
      http::cleanup $token
   }

   return []
}

proc windowsPlatform {} {
   return [expr {$::tcl_platform(platform) eq "windows"}]
}
proc unixPlatform {} {
   return [expr {$::tcl_platform(platform) eq "unix"}]
}

# rawGateway вернет IP шлюза, либо -1
proc rawGateway { } {
   set ip -1
   set cmdres ""
   if {[windowsPlatform]} {
      # Windows specific
      set cmd "route print"
      set re {^.*\n\s+0\.0\.0\.0\s+\s+0\.0\.0\.0\s+(\d+\.\d+\.\d+\.\d+)\s.*$}
   } elseif {[unixPlatform]} {
      # Linux specific
      set cmd "netstat -rn" 
      set re {^.*\n0\.0\.0\.0\s+(\d+.\d+.\d+.\d+)\s.*$}
   } else {
      #writeLogChangeStatus 1 RED "[strTime] Ошибка: rawGateway: платформа не windows и не unix"
      return -1
   }
   catch "exec $cmd" cmdres
   regexp $re $cmdres skip ip
   if [chkIP $ip] {
      return $ip
   } else {
      return -1
   }
}

proc getWhoAmIInfo {} {
    whoAmI
    return [formatWhoAmIData]
}

# Забирает данные у сервиса WhoAmI
proc whoAmI {} {
    global whoami botikkey
    array unset whoami
    set botikkey(time) ""
    catch {http::geturl $botikkey(url_whoami) -handler whoAmIHandler -timeout 5000}
}

# Обработчик соединения с сервисом WhoAmI
proc whoAmIHandler {socket token} {
    global whoami botikkey
    gets $socket line
    set spl   [split $line ":"]
    set kword [lindex $spl 1]
    set value [string range $line [expr [string length $kword] + 2] end]
    set whoami($kword) $value
    if {$kword eq "Time"} {
        set botikkey(time) [ftime]
    }
    return
}

# Отформатированная локальная дата
proc ftime {{t {}}} {
    if {$t eq ""} {set t [clock seconds]}
    return [clock format $t -format "%Y/%m/%d %X"]
}

# Форматирование WhoAmI-данных
proc formatWhoAmIData {} {
    global whoami botikkey
    set myinfo ""
    if {[array exists whoami]} {
        set all [array get whoami]
        foreach {elt val} $all {
            if {$elt eq ""} {continue}
            set myinfo "$myinfo\n:$elt:$val"
            if {$elt eq "Time"} {set myinfo "$myinfo\n:Client time:$botikkey(time)"}
        }
    }
    return $myinfo
}
# Determines whether we have Windows NT kernel v.6.0 or later
proc WinNT6 {} {
    return [expr {[windowsPlatform] && $::tcl_platform(osVersion) >= 6.0}]
}
# Determines whether we have Windows 98 kernel v.4.1 or earlier
proc Win98 {} {
    return [expr {[windowsPlatform] && $::tcl_platform(osVersion) <= 4.10}]
}
proc tokoi {str} {
    return [encoding convertto koi8-r $str]
}
proc dbg {str} {
    set f [open debug.txt a]
    puts $f $str
    close $f
}

package require tile
namespace import -force ttk::*
